/*
 * Main.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import com.topcoder.util.commandline.ArgumentValidationException;
import com.topcoder.util.commandline.CommandLineUtility;
import com.topcoder.util.commandline.IllegalSwitchException;
import com.topcoder.util.commandline.Switch;
import com.topcoder.util.commandline.UsageException;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.ConfigManagerException;

import java.io.File;
import java.io.IOException;
import java.io.PrintStream;

import java.util.Iterator;
import java.util.List;


/**
 * <p>
 * The command line entry point of the File Statistics utility. It uses ExtendedFileStatistics to fulfill the task.
 * When argument is invalid help message should be printed.
 * </p>
 * 
 * <p>
 * Here is the usage of the command. For details, please refer to the Component Specification.
 * <pre>
 * java Main [-c config-file] [-n namespace] [-b base-path] [-r] [-o output-file] path [path2 path3 ...]
 * -c config-file: provides a config file to initialize from, should be in XML format.
 * -n namespace: provides a namespace to initialize from.
 * If neither of these is provided the default namespace is used. Provided namespace should be preloaded.
 * -b base-path: all paths will be relative to it. If missing, the reporting is based on absolute path.
 * -r: whether to recursively traverse sub-directories.
 * -o output-file: the report filename. If not specified, output is printed to console.
 * path [path2 path3 ...]: a list of files or directories to process.
 * </pre>
 * </p>
 * 
 * <p>
 * This class is stateless and thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class Main {
    /**
     * <p>
     * The namespace to use if no namespace is provided.
     * </p>
     */
    public static final String NAMESPACE = "com.topcoder.file.statistics.Main";

    /**
     * <p>
     * The name of the config file switch.
     * </p>
     */
    private static final String CONFIG_FILE_SWITCH_NAME = "c";

    /**
     * <p>
     * The name of the namespace switch.
     * </p>
     */
    private static final String NAMESPACE_SWITCH_NAME = "n";

    /**
     * <p>
     * The name of the base path switch.
     * </p>
     */
    private static final String BASE_PATH_SWITCH_NAME = "b";

    /**
     * <p>
     * The name of the recursive switch.
     * </p>
     */
    private static final String RECURSIVE_SWITCH_NAME = "r";

    /**
     * <p>
     * The name of the output file switch.
     * </p>
     */
    private static final String OUTPUT_FILE_SWTICH_NAME = "o";

    /**
     * <p>
     * Private constructor to prevent instantiation.
     * </p>
     */
    private Main() {
    }

    /**
     * <p>
     * The entry of the command line. Refer to the class level documentation for options.
     * </p>
     * 
     * <p>
     * If there is something wrong in parsing the command line arguments, the usage will be printed to console.
     * Otherwise, it collects the file statistics and generates the statistics report. If any error occurs in
     * collecting file statistics, the error message will be printed to console.
     * </p>
     *
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        // prepare the command line parser
        CommandLineUtility clu = createParser();

        try {
            // parse the command line arguments
            clu.parse(args);
        } catch (UsageException e) {
            // if anything gets wrong, print the usage to the user
            printHelp();
            return;
        } catch (ArgumentValidationException e) {
            // should not happen
        }

        try {
            // process the command for file statistics
            process(clu);
        } catch (Exception e) {
            // if any error occurs (including runtime), print the error message to the user
            System.err.println(e.getMessage());
            return;
        }
    }

    /**
     * <p>
     * Prints help messages to console, similar to the class level documentation. The usage is printed when there is
     * something wrong in parsing the command line arguments.
     * </p>
     */
    private static void printHelp() {
        PrintStream out = System.out;

        out.println("java Main [-" + CONFIG_FILE_SWITCH_NAME + " config-file] [-" + NAMESPACE_SWITCH_NAME
            + " namespace] [-" + BASE_PATH_SWITCH_NAME + " base-path] [-" + RECURSIVE_SWITCH_NAME + "] [-"
            + OUTPUT_FILE_SWTICH_NAME + " output-file] path [path2 path3 ...]");
        out.println("-" + CONFIG_FILE_SWITCH_NAME
            + " config-file: provides a config file to initialize from, should be in XML format.");
        out.println("-" + NAMESPACE_SWITCH_NAME + " namespace: provides a namespace to initialize from.");
        out.println(
            "If neither of these is provided the default namespace is used. Provided namespace should be preloaded.");
        out.println("-" + BASE_PATH_SWITCH_NAME
            + " base-path: all paths will be relative to it. If missing, the reporting is based on absolute path.");
        out.println("-" + RECURSIVE_SWITCH_NAME + ": whether to recursively traverse sub-directories.");
        out.println("-" + OUTPUT_FILE_SWTICH_NAME
            + " output-file: the report filename. If not specified, output is printed to console.");
        out.println("path [path2 path3 ...]: a list of files or directories to process.");
    }

    /**
     * <p>
     * Creates the command line utility to parse the command line arguments. It should have at least one parameter and
     * five optional switches.
     * </p>
     *
     * @return the command line utility
     */
    private static CommandLineUtility createParser() {
        // the command should have at least one parameter
        CommandLineUtility clu = new CommandLineUtility(false, 1, -1);

        try {
            // add the optional config file switch (one argument)
            clu.addSwitch(new Switch(CONFIG_FILE_SWITCH_NAME, false, 1, 1, null));

            // add the optional namespace switch (one argument)
            clu.addSwitch(new Switch(NAMESPACE_SWITCH_NAME, false, 1, 1, null));

            // add the optional base path switch (one argument)
            clu.addSwitch(new Switch(BASE_PATH_SWITCH_NAME, false, 1, 1, null));

            // add the optional recursive switch (no argument)
            clu.addSwitch(new Switch(RECURSIVE_SWITCH_NAME, false, 0, 0, null));

            // add the optional output file switch (one argument)
            clu.addSwitch(new Switch(OUTPUT_FILE_SWTICH_NAME, false, 1, 1, null));
        } catch (IllegalSwitchException e) {
            // should not happen
        }
        return clu;
    }

    /**
     * <p>
     * Collects the file statistics and generates the statistics report, according to the command parameters specified
     * in the given clu. It uses ExtendedFileStatistics to fulfill the task.
     * </p>
     * 
     * <p>
     * The command line utility given should be parsed against the command line arguments. This method just propagates
     * the exceptions to the caller so that the caller can choose what to do with them.
     * </p>
     *
     * @param clu the command line utility with switch arguments and parameters
     *
     * @throws ConfigManagerException if config file cannot be loaded into the namespace
     * @throws IOException if I/O operation fails
     * @throws FileStatisticsException if any error occurs in collecting file statistics
     */
    private static void process(CommandLineUtility clu)
        throws ConfigManagerException, IOException, FileStatisticsException
    {
        // get all the valid switches and parameters
        List switches = clu.getValidSwitches();
        List parameters = clu.getParameters();

        String configFile = null;
        String namespace = null;
        String basePath = null;
        boolean recursive = false;
        String outputFile = null;

        // fill the stats parameters with the switch arguments
        for (Iterator i = switches.iterator(); i.hasNext();) {
            Switch s = (Switch) i.next();
            String name = s.getName();
            String value = s.getValue();

            if (name.equals(CONFIG_FILE_SWITCH_NAME)) {
                configFile = value;
            } else if (name.equals(NAMESPACE_SWITCH_NAME)) {
                namespace = value;
            } else if (name.equals(BASE_PATH_SWITCH_NAME)) {
                basePath = value;
            } else if (name.equals(RECURSIVE_SWITCH_NAME)) {
                recursive = true;
            } else if (name.equals(OUTPUT_FILE_SWTICH_NAME)) {
                outputFile = value;
            }
        }

        // use the default namespace if not specified
        if (namespace == null) {
            namespace = NAMESPACE;
        }

        // load the config file into the namespace if specified
        if (configFile != null) {
            ConfigManager.getInstance().add(namespace, configFile, ConfigManager.CONFIG_XML_FORMAT);
        }

        // use ExtendedFileStatistics to process files and directories
        ExtendedFileStatistics stats = new ExtendedFileStatistics(namespace);

        // set the base directory of the reporting if specified
        if (basePath != null) {
            stats.getReporting().reset(new File(basePath));
        }

        // process files and directories, applying the recursive flag if specified
        for (Iterator i = parameters.iterator(); i.hasNext();) {
            File file = new File((String) i.next());

            if (file.isDirectory()) {
                stats.processDirectory(file, recursive);
            } else {
                stats.processFile(file);
            }
        }

        if (outputFile == null) {
            // if output file is not specified, print the report to console
            stats.printReport();
        } else {
            // otherwise, write the report to the output file
            stats.printReport(new File(outputFile));
        }
    }
}
