/*
 * BasicStatsReporting.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics.reporting;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;

import com.topcoder.util.config.Property;

import java.util.Iterator;


/**
 * <p>
 * BasicStatsReporting is a reporting implementation that renders user-friendly reports. The report consists of four
 * sections: file, directory, alias and overall. It shows the value of each metric for each entry in each of the
 * sections. Here is an example:
 * <pre>
 * File Statistics
 * ---------------
 * File name: a\p.txt
 *     line: 6
 * File name: a\q.java
 *     line: 18
 * File name: b\test.sql
 *     line: 22
 *
 * Directory Statistics
 * --------------------
 * Directory name: a
 *     line: 24
 * Directory name: b
 *     line: 22
 *
 * Alias Statistics
 * ----------------
 * Alias name: Text File
 *     line: 28
 * Alias name: Source File
 *     line: 18
 *
 * Overall Statistics
 * ------------------
 *     line: 46
 * </pre>
 * </p>
 * 
 * <p>
 * It does not need any configuration. However constructor is maintained to accept property argument for reflection
 * purpose.
 * </p>
 * 
 * <p>
 * This class is not thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class BasicStatsReporting extends AbstractStatsReporting {
    /**
     * <p>
     * The character to form the underline beneath the title.
     * </p>
     */
    private static final char UNDERLINE_CHAR = '-';

    /**
     * <p>
     * The String representing the line feed which is system-dependent.
     * </p>
     */
    private static final String LINE_FEED = System.getProperty("line.separator");

    /**
     * <p>
     * The String to separate the name and value on each line of the report.
     * </p>
     */
    private static final String VALUE_SEPARATOR = ": ";

    /**
     * <p>
     * The indentation of the metric for pretty formatting.
     * </p>
     */
    private static final String METRIC_INDENTATION = "    ";

    /**
     * <p>
     * The suffix String to append to the stats name to form the entry name.
     * </p>
     */
    private static final String NAME_SUFFIX = " name";

    /**
     * <p>
     * The suffix String to append to the stats name to form the title name.
     * </p>
     */
    private static final String TITLE_SUFFIX = " Statistics";

    /**
     * <p>
     * The name of the file statistics kept in this reporting.
     * </p>
     */
    private static final String FILE_STATS_NAME = "File";

    /**
     * <p>
     * The name of the directory statistics kept in this reporting.
     * </p>
     */
    private static final String DIRECTORY_STATS_NAME = "Directory";

    /**
     * <p>
     * The name of the alias statistics kept in this reporting.
     * </p>
     */
    private static final String ALIAS_STATS_NAME = "Alias";

    /**
     * <p>
     * The name of the overall statistics kept in this reporting.
     * </p>
     */
    private static final String OVERALL_STATS_NAME = "Overall";

    /**
     * <p>
     * Default constructor. Does nothing.
     * </p>
     */
    public BasicStatsReporting() {
    }

    /**
     * <p>
     * Constructor with property. Simply propagates it to super constructor.
     * </p>
     *
     * @param property the property to configure from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if configuration error occurs
     */
    public BasicStatsReporting(Property property) throws ConfigurationException {
        super(property);
    }

    /**
     * <p>
     * This generates a user-readable report. Refer to the class documentation for the report format.
     * </p>
     *
     * @return the generated report
     */
    public String generateReport() {
        StringBuffer sb = new StringBuffer();

        // write the file stats section
        writeStatsCollection(sb, FILE_STATS_NAME, getFileStats());
        sb.append(LINE_FEED);

        // write the directory stats section
        writeStatsCollection(sb, DIRECTORY_STATS_NAME, getDirectoryStats());
        sb.append(LINE_FEED);

        // write the alias stats section
        writeStatsCollection(sb, ALIAS_STATS_NAME, getAliasStats());
        sb.append(LINE_FEED);

        // write the overall stats section
        writeStatsResult(sb, OVERALL_STATS_NAME, getOverallStats());

        return sb.toString();
    }

    /**
     * <p>
     * Writes the stats heading and the value of each metric for each entry in the collection into the given
     * StringBuffer. The key of each entry will be shown. The metric lines are indented for pretty formatting.
     * </p>
     *
     * @param sb the StringBuffer to write to
     * @param statsName the name of the stats
     * @param stats the statistical collection to write
     */
    private void writeStatsCollection(StringBuffer sb, String statsName, StatsCollection stats) {
        // write the heading (with title and underline)
        writeHeading(sb, statsName);

        String name = statsName + NAME_SUFFIX + VALUE_SEPARATOR;

        for (Iterator i = stats.getAllKeys().iterator(); i.hasNext();) {
            String key = (String) i.next();

            // write the line with name and key
            sb.append(name);
            sb.append(key);
            sb.append(LINE_FEED);

            // write the value for each metric
            writeStatsResult(sb, stats.get(key));
        }
    }

    /**
     * <p>
     * Writes the stats heading and the values for each metric specified in the result into the given StringBuffer.
     * </p>
     *
     * @param sb the StringBuffer to write to
     * @param statsName the name of the stats
     * @param result the statistical result to write
     */
    private void writeStatsResult(StringBuffer sb, String statsName, StatsResult result) {
        // write the heading (with title and underline)
        writeHeading(sb, statsName);

        // write the value for each metric
        writeStatsResult(sb, result);
    }

    /**
     * <p>
     * Writes the values for each metric specified in the result into the given StringBuffer. Each metric is listed on
     * a line. The line is indented for pretty formatting.
     * </p>
     *
     * @param sb the StringBuffer to write to
     * @param result the statistical result to write
     */
    private void writeStatsResult(StringBuffer sb, StatsResult result) {
        for (Iterator i = result.getMetrics().iterator(); i.hasNext();) {
            Metric metric = (Metric) i.next();

            // the line is indented for pretty formatting
            sb.append(METRIC_INDENTATION);

            // write the display name of the metric with its value
            sb.append(metric.getDisplayName());
            sb.append(VALUE_SEPARATOR);
            sb.append(result.getStat(metric));
            sb.append(LINE_FEED);
        }
    }

    /**
     * <p>
     * Writes the stats heading into the given StringBuffer. The heading consists of the title line, followed by an
     * underline of the same length.
     * </p>
     *
     * @param sb the StringBuffer to write to
     * @param statsName the name of the stats
     */
    private void writeHeading(StringBuffer sb, String statsName) {
        String title = statsName + TITLE_SUFFIX;

        // write the title line
        sb.append(title);
        sb.append(LINE_FEED);

        // write the underline (same length as the title)
        for (int i = 0; i < title.length(); i++) {
            sb.append(UNDERLINE_CHAR);
        }
        sb.append(LINE_FEED);
    }
}
