/*
 * AbstractStatsReportingTest.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.matcher.TextFileMatcher;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;
import com.topcoder.file.statistics.reporting.AbstractStatsReporting;
import com.topcoder.file.statistics.reporting.StatsCollection;

import com.topcoder.util.config.Property;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;


/**
 * Unit tests for AbstractStatsReporting implementation.
 * 
 * <p>
 * Note that the getter methods are not explicitly tested here. They are tested inside the aggregate() method, which
 * produces some side effects to the stats collection and stats result returned.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class AbstractStatsReportingTest extends TestCase {
    /**
     * The path of the file to be processed and reported.
     */
    private static final String FILE = "test_files/a/p.txt";

    /**
     * The path of the base directory used in the reporting.
     */
    private static final String BASE_DIRECTORY = "test_files/";

    /**
     * The AbstractStatsReporting instance to test against.
     */
    private AbstractStatsReporting reporting = null;

    /**
     * The processor that generates the result for the reporting.
     */
    private FileProcessor processor = null;

    /**
     * The file to be processed and reported.
     */
    private File file = null;

    /**
     * The base directory used in the reporting.
     */
    private File baseDirectory = null;

    /**
     * The processing result to be aggregated.
     */
    private StatsResult result = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(AbstractStatsReportingTest.class);

        return suite;
    }

    /**
     * Prepares an AbstractStatsReporting instance for testing. Also prepares the associated processor, file,
     * baseDirectory, and the processing result.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void setUp() throws Exception {
        reporting = new SimpleAbstractStatsReporting();
        processor = new TextLineCounter("alias", new TextFileMatcher());
        file = new File(FILE);
        baseDirectory = new File(BASE_DIRECTORY);
        result = processor.process(file);
    }

    /**
     * Tests the accessibility of default constructor.
     */
    public void testConstructor_Default() {
        new SimpleAbstractStatsReporting();
    }

    /**
     * Test of constructor with null property. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_NullProperty() throws Exception {
        try {
            new SimpleAbstractStatsReporting(null);
            fail("Creates AbstractStatsReporting with null property");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of aggregate method with null processor. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testAggregate_NullProcessor() throws Exception {
        try {
            reporting.aggregate(null, file, result);
            fail("Aggregates result with null processor");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of aggregate method with null file. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testAggregate_NullFile() throws Exception {
        try {
            reporting.aggregate(processor, null, result);
            fail("Aggregates result with null file");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of aggregate method with null result. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testAggregate_NullResult() throws Exception {
        try {
            reporting.aggregate(processor, file, null);
            fail("Aggregates result with null result");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of aggregate method with bad file not in base directory. Expects StatsReportingException.
     */
    public void testAggregate_BadFile() {
        // set the base directory
        reporting.reset(baseDirectory);

        try {
            reporting.aggregate(processor, new File("build.xml"), result);
            fail("Aggregates result with file not in base directory");
        } catch (StatsReportingException e) {
            // good
        }
    }

    /**
     * Test of aggregate method with no base directory. Verifies if the result is aggregated in file stats, directory
     * stats, alias stats and overall stats. Also verifies if the file path and directory path are absolute.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testAggregate_NoBaseDir() throws Exception {
        reporting.aggregate(processor, file, result);

        // verify the file stats
        StatsCollection fileStats = reporting.getFileStats();

        assertTrue("Fails to aggregate file stats", TestHelper.equals(result, fileStats.get(file.getCanonicalPath())));

        // verify the directory stats
        StatsCollection directoryStats = reporting.getDirectoryStats();
        File parent = file.getCanonicalFile().getParentFile();

        while (parent != null) {
            assertTrue("Fails to aggregate directory stats",
                TestHelper.equals(result, directoryStats.get(parent.getCanonicalPath())));
            parent = parent.getParentFile();
        }

        // verify the alias stats
        StatsCollection aliasStats = reporting.getAliasStats();

        assertTrue("Fails to aggregate alias stats", TestHelper.equals(result, aliasStats.get(processor.getAlias())));

        // verify the overall stats
        StatsResult overallStats = reporting.getOverallStats();

        assertTrue("Fails to aggregate overall stats", TestHelper.equals(result, overallStats));
    }

    /**
     * Test of aggregate method with base directory. Verifies if the result is aggregated in file stats, directory
     * stats, alias stats and overall stats. Also verifies if the file path and directory path are relative to base
     * directory.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testAggregate_BaseDir() throws Exception {
        String basePath = baseDirectory.getCanonicalPath();
        int stripIndex = basePath.length() + 1;

        // set the base directory
        reporting.reset(baseDirectory);

        reporting.aggregate(processor, file, result);

        // verify the file stats
        StatsCollection fileStats = reporting.getFileStats();

        assertTrue("Fails to aggregate file stats",
            TestHelper.equals(result, fileStats.get(file.getCanonicalPath().substring(stripIndex))));

        // verify the directory stats
        StatsCollection directoryStats = reporting.getDirectoryStats();
        File parent = file.getCanonicalFile().getParentFile();

        while (!parent.getCanonicalPath().equals(basePath)) {
            assertTrue("Fails to aggregate directory stats",
                TestHelper.equals(result, directoryStats.get(parent.getCanonicalPath().substring(stripIndex))));
            parent = parent.getParentFile();
        }

        // verify the alias stats
        StatsCollection aliasStats = reporting.getAliasStats();

        assertTrue("Fails to aggregate alias stats", TestHelper.equals(result, aliasStats.get(processor.getAlias())));

        // verify the overall stats
        StatsResult overallStats = reporting.getOverallStats();

        assertTrue("Fails to aggregate overall stats", TestHelper.equals(result, overallStats));
    }

    /**
     * Test of reset method with null base directory. Verifies it by getting the null base directory back after
     * resetting it. Also verifies if all the stats are cleared.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testReset_NullBaseDirectory() throws Exception {
        // aggregate some result
        reporting.aggregate(processor, file, result);

        reporting.reset(null);
        assertNull("Fails to reset base directory", reporting.getBaseDirectory());
        assertTrue("Fails to clear file stats", reporting.getFileStats().getAllKeys().isEmpty());
        assertTrue("Fails to clear directory stats", reporting.getDirectoryStats().getAllKeys().isEmpty());
        assertTrue("Fails to clear alias stats", reporting.getAliasStats().getAllKeys().isEmpty());
        assertTrue("Fails to clear overall stats", reporting.getOverallStats().getMetrics().isEmpty());
    }

    /**
     * Test of reset method with non-null base directory. Verifies it by getting the non-null base directory back after
     * resetting it. Also verifies if all the stats are cleared.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testReset_NonNullBaseDirectory() throws Exception {
        // aggregate some result
        reporting.aggregate(processor, file, result);

        reporting.reset(baseDirectory);
        assertEquals("Fails to reset base directory", baseDirectory, reporting.getBaseDirectory());
        assertTrue("Fails to clear file stats", reporting.getFileStats().getAllKeys().isEmpty());
        assertTrue("Fails to clear directory stats", reporting.getDirectoryStats().getAllKeys().isEmpty());
        assertTrue("Fails to clear alias stats", reporting.getAliasStats().getAllKeys().isEmpty());
        assertTrue("Fails to clear overall stats", reporting.getOverallStats().getMetrics().isEmpty());
    }

    /**
     * A simple implementation of the AbstractStatsReporting class for testing purpose. The constructors just propagate
     * to the super constructors. The generateReport() method does nothing.
     */
    private class SimpleAbstractStatsReporting extends AbstractStatsReporting {
        /**
         * Default constructor. Just propagates to the super constructor.
         */
        public SimpleAbstractStatsReporting() {
            super();
        }

        /**
         * This constructor loads configuration from the property. Just propagates to the super constructor.
         *
         * @param property the property to configure from
         *
         * @throws NullPointerException if property is null
         * @throws ConfigurationException if any error occurs in configuring the processor
         */
        public SimpleAbstractStatsReporting(Property property)
            throws ConfigurationException
        {
            super(property);
        }

        /**
         * <p>
         * Generates the report with all the aggregated results.
         * </p>
         *
         * @return always null
         */
        public String generateReport() {
            return null;
        }
    }
}
