/*
 * ExtendedFileStatisticsTest.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.matcher.ExtensionMatcher;
import com.topcoder.file.statistics.processor.linecounter.CStyleLineCounter;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;
import com.topcoder.file.statistics.reporting.BasicStatsReporting;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.PrintStream;

import java.util.List;


/**
 * Unit tests for ExtendedFileStatistics implementation.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class ExtendedFileStatisticsTest extends TestCase {
    /**
     * The path of the base directory used in the reporting.
     */
    private static final String BASE_DIRECTORY = "test_files/";

    /**
     * The path of the directory to be processed.
     */
    private static final String DIRECTORY = "test_files/a";

    /**
     * The path of the report file to be printed.
     */
    private static final String PRINT_REPORT_FILE = "test_files/print.txt";

    /**
     * The path of the expected report when recursive is false.
     */
    private static final String NON_RECURSIVE_REPORT_FILE = "test_files/non_recursive.txt";

    /**
     * The path of the expected report when recursive is true.
     */
    private static final String RECURSIVE_REPORT_FILE = "test_files/recursive.txt";

    /**
     * The ExtendedFileStatistics instance to test against.
     */
    private ExtendedFileStatistics stats = null;

    /**
     * The directory to be processed.
     */
    private File directory = null;

    /**
     * The reporting implementation of the FileStatistics instance. (BasicStatsReporting)
     */
    private StatsReporting reporting = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(ExtendedFileStatisticsTest.class);

        return suite;
    }

    /**
     * Prepares an ExtendedFileStatistics instance for testing. The reporting is set to BasicStatsReporting and the
     * matchall flag is set to false. The stats just contains one processor TextLineCounter associated with the an
     * ExtensionMatcher matching files with extension "txt".
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void setUp() throws Exception {
        ConfigHelper.loadConfig();

        FileMatcher matcher = new ExtensionMatcher("txt");
        FileProcessor processor = new TextLineCounter("Text File", matcher);

        reporting = new BasicStatsReporting();
        reporting.reset(new File(BASE_DIRECTORY));
        directory = new File(DIRECTORY);

        stats = new ExtendedFileStatistics();
        stats.setReporting(reporting);
        stats.setMatchAll(false);
        stats.addFileProcessor(processor);
    }

    /**
     * Clears all the namespaces.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void tearDown() throws Exception {
        ConfigHelper.unloadConfig();
    }

    /**
     * Test of constructor with null namespace. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_NullNamespace() throws Exception {
        try {
            new ExtendedFileStatistics(null);
            fail("Creates ExtendedFileStatistics with null namespace");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with empty namespace. Expects IllegalArgumentException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_EmptyNamespace() throws Exception {
        try {
            new ExtendedFileStatistics("");
            fail("Creates ExtendedFileStatistics with empty namespace");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with bad namespace (no processors is present). Expects ConfigurationException.
     */
    public void testConstructor_BadNamespace() {
        try {
            new ExtendedFileStatistics(ConfigHelper.BAD_FILE_STATISTICS_NAMESPACE);
            fail("Creates ExtendedFileStatistics with bad namespace");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * Test of constructor with good namespace. Verifies if two processors TextLineCounter and CStyleLineCounter are
     * loaded, the reporting is of type BasicStatsReporting, and the matchall flag is set to true.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_GoodNamespace() throws Exception {
        stats = new ExtendedFileStatistics(ConfigHelper.GOOD_FILE_STATISTICS_NAMESPACE);

        // verify the processors
        List processors = stats.getAllFileProcessors();

        assertEquals("Fails to load processors", 2, processors.size());
        assertTrue("Fails to load processors", processors.get(0) instanceof TextLineCounter);
        assertTrue("Fails to load processors", processors.get(1) instanceof CStyleLineCounter);

        // verify the reporting
        assertTrue("Fails to load reporting", stats.getReporting() instanceof BasicStatsReporting);

        // verify the matchall flag
        assertTrue("Fails to load matchall flag", stats.isMatchAll());
    }

    /**
     * Test of processDirectory method with null directory. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testProcessDirectory_NullDirectory() throws Exception {
        try {
            stats.processDirectory(null);
            fail("Processes a null directory");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of processDirectory method with invalid directory. Expects IllegalArgumentException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testProcessDirectory_InvalidDirectory()
        throws Exception
    {
        try {
            stats.processDirectory(new File(PRINT_REPORT_FILE));
            fail("Processes an invalid directory");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of processDirectory method with directory in non-recursive mode. Verifies if the directory "test_files/a"
     * was processed from the generated report.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testProcessDirectory_NonRecursive() throws Exception {
        stats.processDirectory(directory, false);
        assertTrue("Fails to process directory",
            TestHelper.verifyReport(new File(NON_RECURSIVE_REPORT_FILE), reporting.generateReport()));
    }

    /**
     * Test of processDirectory method with directory in recursive mode. Verifies if the directories "test_files/a" and
     * "test_files/a/a" were processed from the generated report.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testProcessDirectory_Recursive() throws Exception {
        stats.processDirectory(directory, true);
        assertTrue("Fails to process directory",
            TestHelper.verifyReport(new File(RECURSIVE_REPORT_FILE), reporting.generateReport()));
    }

    /**
     * Test of printReport method with null file. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testPrintReport_NullFile() throws Exception {
        try {
            stats.printReport(null);
            fail("Prints report to null file");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of printReport method to console. First, processes a directory. Verifies if the report was printed to
     * console after calling it.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testPrintReport_Console() throws Exception {
        // direct the system output to byte array for verification
        PrintStream oldOut = System.out;
        ByteArrayOutputStream newOut = new ByteArrayOutputStream();

        System.setOut(new PrintStream(newOut));

        // process a directory
        stats.processDirectory(directory);

        // print report to console
        stats.printReport();

        assertTrue("Fails to print report to console",
            TestHelper.verifyReport(new File(NON_RECURSIVE_REPORT_FILE), newOut.toString()));

        // restore the system output to console
        System.setOut(oldOut);
    }

    /**
     * Test of printReport method to file. First, processes a directory. Verifies if the report was printed to the file
     * after calling it. Removes the printed file on return.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testPrintReport_File() throws Exception {
        File file = new File(PRINT_REPORT_FILE);

        // process a directory
        stats.processDirectory(directory);

        // print report to file
        stats.printReport(file);

        assertTrue("Fails to print report to file", TestHelper.verifyReport(new File(NON_RECURSIVE_REPORT_FILE), file));

        // remove the printed file
        file.delete();
    }
}
