/*
 * MainTest.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import com.topcoder.util.config.ConfigManager;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.PrintStream;


/**
 * Unit tests for Main implementation.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class MainTest extends TestCase {
    /**
     * The path of the directory to be processed.
     */
    private static final String DIRECTORY = "test_files/a";

    /**
     * The path of the file to be processed.
     */
    private static final String FILE = "test_files/a/p.txt";

    /**
     * The config file to be passed in the command line.
     */
    private static final String CONFIG_FILE = "config.xml";

    /**
     * The namespace to be passed in the command line.
     */
    private static final String NAMESPACE = "com.topcoder.file.statistics";

    /**
     * The base path to be passed in the command line.
     */
    private static final String BASE_PATH = "test_files/";

    /**
     * The output file to be passed in the command line.
     */
    private static final String OUTPUT_FILE = "test_files/print.txt";

    /**
     * The path of the expected report when processing with the directory and recursive is false, or just processing
     * with the file.
     */
    private static final String NON_RECURSIVE_REPORT_FILE = "test_files/non_recursive.txt";

    /**
     * The path of the expected report when processing with the directory and recursive is true.
     */
    private static final String RECURSIVE_REPORT_FILE = "test_files/recursive.txt";

    /**
     * The buffer to hold the output to console.
     */
    private ByteArrayOutputStream out = null;

    /**
     * The standard system output stream to be restored after each test.
     */
    private PrintStream sysOut = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(MainTest.class);

        return suite;
    }

    /**
     * Directs the system output to byte array for easy verification. Anything printed to console can be verified using
     * the byte array.
     */
    public void setUp() {
        out = new ByteArrayOutputStream();
        sysOut = System.out;
        System.setOut(new PrintStream(out));
    }

    /**
     * Clears all the namespaces. Restores the system output stream.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void tearDown() throws Exception {
        ConfigHelper.unloadConfig();
        System.setOut(sysOut);
    }

    /**
     * Test of main method on invalid input arguments. Verifies if the usage was printed to console. For simplicity,
     * the usage can be identified by the string "java Main".
     */
    public void testMain_OnInvalidInput() {
        Main.main(new String[0]);
        assertTrue("Fails to print usage to console", out.toString().startsWith("java Main"));
    }

    /**
     * Test of main method on config file argument. Verifies if the config file was loaded to the default namespace.
     */
    public void testMain_OnConfigFile() {
        Main.main(new String[] {"-c", CONFIG_FILE, FILE});
        assertTrue("Fails to load config file to namespace", ConfigManager.getInstance().existsNamespace(Main.NAMESPACE));
    }

    /**
     * Test of main method on namespace argument. Specifies the config file at the same time. Verifies if the config
     * file was loaded to the specified namespace.
     */
    public void testMain_OnNamespace() {
        Main.main(new String[] {"-c", CONFIG_FILE, "-n", NAMESPACE, FILE});
        assertTrue("Fails to load config file to namespace", ConfigManager.getInstance().existsNamespace(NAMESPACE));
    }

    /**
     * Test of main method on no base path argument. Processes a file. Verifies if the file path and directory path are
     * all absolute from the generated report.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testMain_OnNoBasePath() throws Exception {
        Main.main(new String[] {"-c", CONFIG_FILE, FILE});
        assertFalse("Fails to set the base path",
            TestHelper.verifyReport(new File(NON_RECURSIVE_REPORT_FILE), out.toString()));
    }

    /**
     * Test of main method on base path argument. Processes a file. Verifies if the file path and directory path are
     * relative to base path from the generated report.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testMain_OnBasePath() throws Exception {
        Main.main(new String[] {"-c", CONFIG_FILE, "-b", BASE_PATH, FILE});
        assertTrue("Fails to set the base path",
            TestHelper.verifyReport(new File(NON_RECURSIVE_REPORT_FILE), out.toString()));
    }

    /**
     * Test of main method on no recursive argument. Processes a directory. Verifies if the sub-directories are not
     * processed from the generated report.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testMain_OnNoRecursive() throws Exception {
        Main.main(new String[] {"-c", CONFIG_FILE, "-b", BASE_PATH, DIRECTORY});
        assertTrue("Fails to set recursive flag",
            TestHelper.verifyReport(new File(NON_RECURSIVE_REPORT_FILE), out.toString()));
    }

    /**
     * Test of main method on recursive argument. Processes a directory. Verifies if the sub-directories are processed
     * from the generated report.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testMain_OnRecursive() throws Exception {
        Main.main(new String[] {"-c", CONFIG_FILE, "-b", BASE_PATH, "-r", DIRECTORY});
        assertTrue("Fails to set recursive flag",
            TestHelper.verifyReport(new File(RECURSIVE_REPORT_FILE), out.toString()));
    }

    /**
     * Test of main method on no output file argument. Processes a file. Verifies if the report was printed to console.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testMain_OnNoOutputFile() throws Exception {
        Main.main(new String[] {"-c", CONFIG_FILE, "-b", BASE_PATH, FILE});
        assertTrue("Fails to print report to console",
            TestHelper.verifyReport(new File(NON_RECURSIVE_REPORT_FILE), out.toString()));
    }

    /**
     * Test of main method on output file argument. Processes a file. Verifies if the report was printed to output
     * file. Removes the printed file on return.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testMain_OnOutputFile() throws Exception {
        Main.main(new String[] {"-c", CONFIG_FILE, "-b", BASE_PATH, "-o", OUTPUT_FILE, FILE});

        File file = new File(OUTPUT_FILE);

        assertTrue("Fails to print report to output file",
            TestHelper.verifyReport(new File(NON_RECURSIVE_REPORT_FILE), file));
        file.delete();
    }
}
