/*
 * TextLineCounterTest.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.matcher.TextFileMatcher;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;

import java.util.List;


/**
 * Unit tests for TextLineCounter implementation.
 * 
 * <p>
 * Here we assume that the constructor with a Property has the same effect as specifying the alias and the matcher. The
 * TextLineCounter instance to test against is only constructed using the latter method.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class TextLineCounterTest extends TestCase {
    /**
     * The file to be processed by the TextLineCounter instance.
     */
    private static final String FILE = "test_files/a/p.txt";

    /**
     * The number of lines in the testing file.
     */
    private static final long LINE_COUNT = 10;

    /**
     * The TextLineCounter instance to test against.
     */
    private TextLineCounter processor = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(TextLineCounterTest.class);

        return suite;
    }

    /**
     * Prepares a TextLineCounter instance for testing.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void setUp() throws Exception {
        ConfigHelper.loadConfig();
        processor = new TextLineCounter("alias", new TextFileMatcher());
    }

    /**
     * Clears all the namespaces.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void tearDown() throws Exception {
        ConfigHelper.unloadConfig();
    }

    /**
     * Test of constructor with null alias. Expects NullPointerException.
     */
    public void testConstructor_NullAlias() {
        try {
            new TextLineCounter(null, new TextFileMatcher());
            fail("Creates TextLineCounter with null alias");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with empty alias. Expects IllegalArgumentException.
     */
    public void testConstructor_EmptyAlias() {
        try {
            new TextLineCounter("", new TextFileMatcher());
            fail("Creates TextLineCounter with empty alias");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with null matcher. Expects NullPointerException.
     */
    public void testConstructor_NullMatcher() {
        try {
            new TextLineCounter("alias", null);
            fail("Creates TextLineCounter with null matcher");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with null property. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_NullProperty() throws Exception {
        try {
            new TextLineCounter(null);
            fail("Creates TextLineCounter with null property");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with bad property (no alias property). Expects ConfigurationException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_BadProperty() throws Exception {
        try {
            new TextLineCounter(ConfigHelper.getRootProperty(ConfigHelper.BAD_FILE_PROCESSOR_BASE_NAMESPACE));
            fail("Creates TextLineCounter with bad property");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * Test of getSupportedMetrics method. Verifies if it returns a list containing single Metric composed of the NAME,
     * DISPLAY_NAME and a description.
     */
    public void testGetSupportedMetrics() {
        List metrics = processor.getSupportedMetrics();

        assertEquals("Returns an incorrect metric list", 1, metrics.size());

        Metric metric = (Metric) metrics.get(0);

        assertEquals("Returns an incorrect metric name", TextLineCounter.NAME, metric.getName());
        assertEquals("Returns an incorrect metric display name", TextLineCounter.DISPLAY_NAME, metric.getDisplayName());
        assertNotNull("Returns an incorrect metric description", metric.getDescription());
    }

    /**
     * Test of process method with null file. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testProcess_NullFile() throws Exception {
        try {
            processor.process(null);
            fail("Processes a null file");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of process method with non-null file. Verifies if it returns a StatsResult containing the number of lines
     * under the metric of "Line Count".
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testProcess_NonNullFile() throws Exception {
        StatsResult result = processor.process(new File(FILE));
        Metric metric = (Metric) processor.getSupportedMetrics().get(0);

        assertEquals("Fails to process the file", LINE_COUNT, result.getStat(metric));
    }
}
