/*
 * StatsReporting.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import java.io.File;


/**
 * <p>
 * StatsReporting interface defines the contract to aggregate and render the statistical report. Result is fed into
 * this interface one by one and the file report is then generated. There is a way to reset (clear) the current
 * aggregated results.
 * </p>
 * 
 * <p>
 * The base directory defines the directory where all the processed files are under. However it is optional. If not
 * specified, it means that absolute paths are desirable in the reporting.
 * </p>
 * 
 * <p>
 * All implementations should have a constructor with a Property as an argument in order to reflect from configuration
 * file. This requirement can be dropped if it's not necessary to use this way.
 * </p>
 * 
 * <p>
 * There is no thread-safety requirement or recommendation for the implementations.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0c
 */
public interface StatsReporting {
    /**
     * <p>
     * Aggregates the result into the reporting implementation. The results are kept by the reporting itself.
     * </p>
     *
     * @param processor the processor that generated the result
     * @param filePath the file that was processed
     * @param result the processing result
     *
     * @throws NullPointerException if any argument is null
     * @throws IllegalArgumentException if filePath is empty
     * @throws StatsReportingException if baseDirectory is not null but filePath is not in base directory, or if error
     *         occurs in aggregation
     */
    void aggregate(FileProcessor processor, String filePath, StatsResult result)
        throws StatsReportingException;

    /**
     * <p>
     * Generates the report with all the aggregated results.
     * </p>
     *
     * @return the generated report
     *
     * @throws StatsReportingException if error occurs in generating report
     */
    String generateReport() throws StatsReportingException;

    /**
     * <p>
     * Clears all the aggregated results and sets the base directory (which is nullable).
     * </p>
     *
     * @param baseDirectory the base directory to reset
     *
     * @throws IllegalArgumentException if baseDirectory is not null but is not a valid directory
     */
    void reset(File baseDirectory);
}
