/*
 * ZipExtractor.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics.extractor;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.ArchiveExtractingException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.util.config.Property;
import com.topcoder.util.archiving.ArchiveUtility;
import com.topcoder.util.archiving.ZipArchiver;
import java.io.File;
import java.io.IOException;

/**
 * <p>
 * Zip implementation of ArchiveExtractor in this release to delegate the logic to TopCoder Compression
 * Utility.
 * </p>
 *
 * <p>
 * It supports the configuration as specified by its base class.
 * </p>
 *
 * <p>
 * This class is thread-safe. Each operation works with a new ArchiveUtility instance.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0c
 * @since 2.0c
 */
public class ZipExtractor extends ArchiveExtractorBase {
    
    /**
     * <p>
     * Constructor with all arguments propagated to base constructor.
     * </p>
     *
     *
     * @param matcher the file matcher
     * @param pathSeparator the path separator
     * @throws NullPointerException if matcher is null
     */
    public  ZipExtractor(FileMatcher matcher, char pathSeparator) {
        super(matcher, pathSeparator);
    }
    
    /**
     * <p>
     * Does nothing more than propagate to base constructor.
     * </p>
     *
     *
     * @param property the property to configure from
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if any error occurs in configuring the extractor
     */
    public  ZipExtractor(Property property) throws ConfigurationException {
        super(property);
    }
    
    /**
     * <p>
     * Extracts file from the archive file. The logic is already well encapsulated by the Compression Utility.
     * </p>
     *
     *
     * @param source the source archive file
     * @param target the target directory
     * @throws NullPointerException if either source or target is null
     * @throws IOException if I/O operation fails (including source/target not exist or target is not a directory)
     * @throws ArchiveExtractingException to encapsulate any implementation specific errors. It is not thrown here.
     */
    public void extract(File source, File target) throws IOException, ArchiveExtractingException {
        if (source == null) {
            throw new NullPointerException("source is null");
        }
        if (target == null) {
            throw new NullPointerException("target is null");
        }
        
        // extract the Zip file
        ArchiveUtility au = new ArchiveUtility(source, new ZipArchiver());
        
        try {
            au.extractContents(target);
        } catch (IllegalArgumentException e) {
            // if target does not exist or is not directory, throw an IOException
            throw new IOException(e.getMessage());
        }
    }
}
