/*
 * RegexMatcher.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.ConfigurationException;

import com.topcoder.util.config.Property;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;


/**
 * <p>
 * Concrete implementation of the FileNameMatcher class that matches file names with one or more regular expressions.
 * The regular expression is defined by the java.util.regex package in JDK 1.4. The matching is based on a subsequence
 * of the input file name, for example, "abc.txt" matches the regular expression "c".
 * </p>
 * 
 * <p>
 * To follow the reflection contract, this class can be constructed via a Property instance. The supported property is
 * named "patterns" which is potentially multi-valued.
 * </p>
 * 
 * <p>
 * This class is stateless and thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class RegexMatcher extends FileNameMatcher {
    /**
     * <p>
     * Configuration key that contains multiple regular expressions to match. The regular expressions are compiled when
     * loaded and invalid pattern will result in a ConfigurationException.
     * </p>
     */
    public static final String PATTERNS_PROPERTY_KEY = "patterns";

    /**
     * <p>
     * A list of compiled regular expressions (Pattern) to match. This list is initialized in constructor and is
     * immutable.
     * </p>
     */
    private List patterns = null;

    /**
     * <p>
     * Constructor with single regular expression pattern.
     * </p>
     *
     * @param pattern the regular expression pattern to match
     *
     * @throws NullPointerException if pattern is null
     */
    public RegexMatcher(Pattern pattern) {
        if (pattern == null) {
            throw new NullPointerException("pattern is null");
        }

        // create a List to hold the single pattern
        patterns = new ArrayList();
        patterns.add(pattern);
    }

    /**
     * <p>
     * Constructor with multiple regular expression patterns. The class will store a copy of the list.
     * </p>
     *
     * @param patterns the multiple regular expression patterns to match
     *
     * @throws NullPointerException if patterns is null
     * @throws IllegalArgumentException if patterns is empty or contains non-Pattern instance
     */
    public RegexMatcher(List patterns) {
        MatcherUtil.checkListForObject(patterns, Pattern.class);

        // the List is fine here, keep a copy of it
        this.patterns = new ArrayList(patterns);
    }

    /**
     * <p>
     * Constructor that loads patterns from a property. It should look for a "patterns" sub-property and extract
     * multiple values. Each property value represents the regular expression of the pattern. The patterns are
     * compiled when loaded and invalid pattern will result in a ConfigurationException.
     * </p>
     * 
     * <p>
     * For details, please refer to the class documentation of java.util.regex.Pattern.
     * </p>
     *
     * @param property the property to configure from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if the "patterns" sub-property is missing or it does not hold any values, or it
     *         contains invalid regular expresssion
     */
    public RegexMatcher(Property property) throws ConfigurationException {
        String[] regex = MatcherUtil.getPropertyValues(property, PATTERNS_PROPERTY_KEY, true);

        // compile regular expressions into patterns
        patterns = new ArrayList();

        for (int i = 0; i < regex.length; i++) {
            try {
                patterns.add(Pattern.compile(regex[i]));
            } catch (PatternSyntaxException e) {
                throw new ConfigurationException("Regular expression " + regex[i] + " is invalid", e);
            }
        }
    }

    /**
     * <p>
     * Checks whether the filename matches any of the patterns defined in the matcher.
     * </p>
     *
     * @param filename the filename to match
     *
     * @return whether the filename matches
     */
    protected boolean match(String filename) {
        // look for the pattern that matches the filename
        for (Iterator i = patterns.iterator(); i.hasNext();) {
            if (((Pattern) i.next()).matcher(filename).find()) {
                return true;
            }
        }

        // all patterns have been exhausted
        return false;
    }
}
