/*
 * CStyleLineCounterTest.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.matcher.TextFileMatcher;
import com.topcoder.file.statistics.processor.linecounter.CStyleLineCounter;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;

import java.util.List;


/**
 * Unit tests for CStyleLineCounter implementation.
 * 
 * <p>
 * Here we assume that the constructor with a Property has the same effect as specifying the alias and the matcher. The
 * CStyleLineCounter instance to test against is only constructed using the latter method.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class CStyleLineCounterTest extends TestCase {
    /**
     * The files to be processed by the CStyleLineCounter instance.
     */
    private static final String[] FILES = {
        "test_files/c/source.cs", "test_files/c/source.java", "test_files/c/source.cpp"
    };

    /**
     * The number of lines in the testing files.
     */
    private static final long[] LINE_COUNTS = {3, 7, 36};

    /**
     * The CStyleLineCounter instance to test against.
     */
    private CStyleLineCounter processor = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(CStyleLineCounterTest.class);

        return suite;
    }

    /**
     * Prepares a CStyleLineCounter instance for testing.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void setUp() throws Exception {
        ConfigHelper.loadConfig();
        processor = new CStyleLineCounter("alias", new TextFileMatcher());
    }

    /**
     * Clears all the namespaces.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void tearDown() throws Exception {
        ConfigHelper.unloadConfig();
    }

    /**
     * Test of constructor with null alias. Expects NullPointerException.
     */
    public void testConstructor_NullAlias() {
        try {
            new CStyleLineCounter(null, new TextFileMatcher());
            fail("Creates CStyleLineCounter with null alias");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with empty alias. Expects IllegalArgumentException.
     */
    public void testConstructor_EmptyAlias() {
        try {
            new CStyleLineCounter("", new TextFileMatcher());
            fail("Creates CStyleLineCounter with empty alias");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with null matcher. Expects NullPointerException.
     */
    public void testConstructor_NullMatcher() {
        try {
            new CStyleLineCounter("alias", null);
            fail("Creates CStyleLineCounter with null matcher");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with null property. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_NullProperty() throws Exception {
        try {
            new CStyleLineCounter(null);
            fail("Creates CStyleLineCounter with null property");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with bad property (no alias property). Expects ConfigurationException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_BadProperty() throws Exception {
        try {
            new CStyleLineCounter(ConfigHelper.getRootProperty(ConfigHelper.BAD_FILE_PROCESSOR_BASE_NAMESPACE));
            fail("Creates CStyleLineCounter with bad property");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * Test of getSupportedMetrics method. Verifies if it returns a list containing single Metric composed of the NAME,
     * DISPLAY_NAME and a description.
     */
    public void testGetSupportedMetrics() {
        List metrics = processor.getSupportedMetrics();

        assertEquals("Returns an incorrect metric list", 1, metrics.size());

        Metric metric = (Metric) metrics.get(0);

        assertEquals("Returns an incorrect metric name", CStyleLineCounter.NAME, metric.getName());
        assertEquals("Returns an incorrect metric display name", CStyleLineCounter.DISPLAY_NAME,
            metric.getDisplayName());
        assertNotNull("Returns an incorrect metric description", metric.getDescription());
    }

    /**
     * Test of process method with null file. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testProcess_NullFile() throws Exception {
        try {
            processor.process(null);
            fail("Processes a null file");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of process method with non-null file. Verifies if it returns a StatsResult containing the number of lines
     * under the metric of "Line Count".
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testProcess_NonNullFile() throws Exception {
        for (int i = 0; i < FILES.length; i++) {
            StatsResult result = processor.process(new File(FILES[i]));
            Metric metric = (Metric) processor.getSupportedMetrics().get(0);

            assertEquals("Fails to process the file", LINE_COUNTS[i], result.getStat(metric));
        }
    }
}
