/*
 * FileTypeMatcherTest.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.matcher.FileTypeMatcher;
import com.topcoder.file.type.FileType;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;


/**
 * Unit tests for FileTypeMatcher implementation.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class FileTypeMatcherTest extends TestCase {
    /**
     * The file for testing the file type.
     */
    private static final String FILE = "test_files/b/test.xml";

    /**
     * The type name of the testing file (specified by configuration).
     */
    private static final String TYPE_NAME = "XML file";

    /**
     * The mime of the testing file (specified by configuration).
     */
    private static final String MIME = "text/xml";

    /**
     * The FileTypeMatcher instance to test against.
     */
    private SimpleFileTypeMatcher matcher = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(FileTypeMatcherTest.class);

        return suite;
    }

    /**
     * Prepares a FileTypeMatcher instance for testing.
     */
    public void setUp() {
        matcher = new SimpleFileTypeMatcher();
    }

    /**
     * Tests the accessibility of default constructor.
     */
    public void testConstructor_Default() {
        new SimpleFileTypeMatcher();
    }

    /**
     * Test of match method with null file. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testMatch_NullFile() throws Exception {
        try {
            matcher.match((File) null);
            fail("Matches a null file");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of match method with non-null file. Verifies if the file type of the file was passed to the abstract
     * match() method by the type name and mime. Also verifies if it returns the result of the abstract match()
     * method.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testMatch_NonNullFile() throws Exception {
        matcher.setMatchResult(true);
        assertTrue("Fails to match file", matcher.match(new File(FILE)));

        FileType type = matcher.getFileType();

        assertEquals("Fails to match file", TYPE_NAME, type.getName());
        assertEquals("Fails to match file", MIME, type.getMime());
    }

    /**
     * A simple implementation of the FileTypeMatcher class for testing purpose. Two additional methods are provided to
     * set the matching result of the match() method, and get the file type passed to the match() method.
     */
    private class SimpleFileTypeMatcher extends FileTypeMatcher {
        /**
         * The matching result of the match() method. It can be changed by its setter method.
         */
        private boolean matchResult = false;

        /**
         * The last file type passed to the match() method. It is updated once match() method is called.
         */
        private FileType type = null;

        /**
         * Default constructor. Does nothing.
         */
        public SimpleFileTypeMatcher() {
            super();
        }

        /**
         * Sets the matching result of the match() method.
         *
         * @param matchResult the matching result of the match() method.
         */
        public void setMatchResult(boolean matchResult) {
            this.matchResult = matchResult;
        }

        /**
         * Returns the last file type passed to the match() method. It can be null if match() method is not called.
         *
         * @return the last file type passed to the match() method.
         */
        public FileType getFileType() {
            return type;
        }

        /**
         * <p>
         * Determines whether the file matches based on the given file type. The result depends on the status set in
         * the setMatchResult() method. In addition, it saves the passed file type.
         * </p>
         *
         * @param type the file type to match
         *
         * @return whether the file type matches
         */
        protected boolean match(FileType type) {
            this.type = type;
            return matchResult;
        }
    }
}
