/*
 * MimeMatcherTest.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.matcher.MimeMatcher;

import com.topcoder.util.config.Property;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;


/**
 * Unit tests for MimeMatcher implementation.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class MimeMatcherTest extends TestCase {
    /**
     * The files to match against by MimeMatcher instances.
     */
    private static final String[] FILES = {"test_files/a/q.class", "test_files/b/test.xml", "test_files/b/test.sql"};

    /**
     * The MimeMatcher instances to test against.
     */
    private MimeMatcher[] matchers = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(MimeMatcherTest.class);

        return suite;
    }

    /**
     * Prepares three MimeMatcher instances for testing. matchers[0] is constructed with a single mime "java/class".
     * matchers[1] is constructed with multiple mimes "java/class", "text/xml". matchers[2] is constructed with
     * property loaded with mimes "java/class", "text/xml".
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void setUp() throws Exception {
        ConfigHelper.loadConfig();

        matchers = new MimeMatcher[3];
        matchers[0] = new MimeMatcher("java/class");
        matchers[1] = new MimeMatcher(Arrays.asList(new String[] {"java/class", "text/xml"}));
        matchers[2] = new MimeMatcher(ConfigHelper.getRootProperty(ConfigHelper.GOOD_MIME_MATCHER_NAMESPACE));
    }

    /**
     * Clears all the namespaces.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void tearDown() throws Exception {
        ConfigHelper.unloadConfig();
    }

    /**
     * Test of constructor with null mime. Expects NullPointerException.
     */
    public void testConstructor_NullMime() {
        try {
            new MimeMatcher((String) null);
            fail("Creates MimeMatcher with null mime");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with empty mime. Expects IllegalArgumentException.
     */
    public void testConstructor_EmptyMime() {
        try {
            new MimeMatcher("");
            fail("Creates MimeMatcher with empty mime");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with null mimes. Expects NullPointerException.
     */
    public void testConstructor_NullMimes() {
        try {
            new MimeMatcher((List) null);
            fail("Creates MimeMatcher with null mimes");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with empty mimes. Expects IllegalArgumentException.
     */
    public void testConstructor_EmptyMimes() {
        try {
            new MimeMatcher(new ArrayList());
            fail("Creates MimeMatcher with empty mimes");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with mimes containing non-String instance. Expects IllegalArgumentException.
     */
    public void testConstructor_NonStringMimes() {
        List mimes = new ArrayList();

        // add a non-String element
        mimes.add(new Object());

        try {
            new MimeMatcher(mimes);
            fail("Creates MimeMatcher with mimes containing non-String instance");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with mimes containing empty String. Expects IllegalArgumentException.
     */
    public void testConstructor_EmptyStringMimes() {
        List mimes = new ArrayList();

        // add an empty String
        mimes.add("");

        try {
            new MimeMatcher(mimes);
            fail("Creates MimeMatcher with mimes containing empty String");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with null property. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_NullProperty() throws Exception {
        try {
            new MimeMatcher((Property) null);
            fail("Creates MimeMatcher with null property");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with bad property (no value is present). Expects ConfigurationException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_BadProperty() throws Exception {
        try {
            new MimeMatcher(ConfigHelper.getRootProperty(ConfigHelper.BAD_MIME_MATCHER_NAMESPACE));
            fail("Creates MimeMatcher with bad property");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * Test of match method with the three files. Expects results for matchers[0]: true, false, false. Expects results
     * for matchers[1]: true, true, false. Expects results for matchers[2]: true, true, false.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testMatch() throws Exception {
        File[] files = new File[FILES.length];

        for (int i = 0; i < FILES.length; i++) {
            files[i] = new File(FILES[i]);
        }

        // test with matchers[0] - only matches java/class file
        assertTrue("Fails to match file", matchers[0].match(files[0]));
        assertFalse("Fails to match file", matchers[0].match(files[1]));
        assertFalse("Fails to match file", matchers[0].match(files[2]));

        // test with matchers[1] - matches java/class, text/xml files
        assertTrue("Fails to match file", matchers[1].match(files[0]));
        assertTrue("Fails to match file", matchers[1].match(files[1]));
        assertFalse("Fails to match file", matchers[1].match(files[2]));

        // test with matchers[2] - matches java/class, text/xml files
        assertTrue("Fails to match file", matchers[2].match(files[0]));
        assertTrue("Fails to match file", matchers[2].match(files[1]));
        assertFalse("Fails to match file", matchers[2].match(files[2]));
    }
}
