/*
 * StatsResultTest.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.util.HashSet;
import java.util.Set;


/**
 * Unit tests for StatsResult implementation.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class StatsResultTest extends TestCase {
    /**
     * The quantity of the metric for testing.
     */
    private static final long RESULT = 100;

    /**
     * The StatsResult instance to test against.
     */
    private StatsResult stats = null;

    /**
     * The metric to set in the StatsResult instance.
     */
    private Metric metric = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(StatsResultTest.class);

        return suite;
    }

    /**
     * Prepares a StatsResult instance for testing. The metric will be set to the StatsResult later.
     */
    public void setUp() {
        stats = new StatsResult();
        metric = new Metric("name", "displayName", "description");
    }

    /**
     * Tests the accessibility of default constructor.
     */
    public void testConstructor_Default() {
        new StatsResult();
    }

    /**
     * Test of constructor with null result. Expects NullPointerException.
     */
    public void testConstructor_NullResult() {
        try {
            new StatsResult(null);
            fail("Creates StatsResult with null result");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with non-null result. Verifies if the result was cloned.
     */
    public void testConstructor_NonNullResult() {
        // make sure an existing metric
        stats.setStat(metric, RESULT);

        StatsResult newStats = new StatsResult(stats);

        assertTrue("Fails to clone the result", TestHelper.equals(stats, newStats));

        stats.setStat(metric, RESULT * 2);

        assertEquals("Fails to clone the result", RESULT, newStats.getStat(metric));
    }

    /**
     * Test of setStat method with null metric. Expects NullPointerException.
     */
    public void testSetStat_NullMetric() {
        try {
            stats.setStat(null, RESULT);
            fail("Sets stat with null metric");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of getStat and setStat methods with non-existing metric. Verifies if the metric was set with the given
     * result.
     */
    public void testGetSetStat_NonExistMetric() {
        stats.setStat(metric, RESULT);
        assertEquals("Returns an incorrect result", RESULT, stats.getStat(metric));
    }

    /**
     * Test of getStat and setStat methods with existing metric. Verifies if the old result of the metric was
     * overwritten.
     */
    public void testGetSetStat_ExistMetric() {
        // make sure an existing metric
        stats.setStat(metric, RESULT * 2);

        stats.setStat(metric, RESULT);
        assertEquals("Returns an incorrect result", RESULT, stats.getStat(metric));
    }

    /**
     * Test of getStat method with null metric. Expects NullPointerException.
     */
    public void testGetStat_NullMetric() {
        try {
            stats.getStat(null);
            fail("Gets stats with null metric");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of getStat method with non-existing metric. Expects IllegalArgumentException.
     */
    public void testGetStat_NonExistMetric() {
        try {
            stats.getStat(new Metric("non-exist", "non-exist", "non-exist"));
            fail("Gets stats with non-existing metric");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of getMetrics method with empty metrics. Verifies if an empty list is returned.
     */
    public void testGetMetrics_Empty() {
        assertTrue("Returns an incorrect metric list", stats.getMetrics().isEmpty());
    }

    /**
     * Test of getMetrics method with non-empty metrics. Verifies if it returns a list containing all the metrics.
     */
    public void testGetMetrics_NonEmpty() {
        Set metrics = new HashSet();

        // make sure non-empty metrics
        for (int i = 0; i < 3; i++) {
            Metric metric = new Metric("name" + i, "displayName" + i, "description" + i);

            metrics.add(metric);
            stats.setStat(metric, i);
        }

        assertEquals("Returns an incorrect metric list", metrics, new HashSet(stats.getMetrics()));
    }

    /**
     * Test of aggregate method with null result. Expects NullPointerException.
     */
    public void testAggregate_NullResult() {
        try {
            stats.aggregate(null);
            fail("Aggregates a null result");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of aggregate method with non-null result. The aggregating result contains two metrics: one already exists
     * in the current result, and the other not. Verifies if the old quantity of the existing metric was added with
     * the quantity from the aggregating result. Also verifies if the non-existing metric was set.
     */
    public void testAggregate_NonNullResult() {
        // make sure an existing metric
        stats.setStat(metric, RESULT);

        StatsResult newStats = new StatsResult(stats);
        Metric newMetric = new Metric("new", "new", "new");

        newStats.setStat(newMetric, RESULT);

        stats.aggregate(newStats);

        assertEquals("Fails to aggregate result", 2, stats.getMetrics().size());
        assertEquals("Fails to aggregate existing metric", RESULT * 2, stats.getStat(metric));
        assertEquals("Fails to aggregate non-existing metric", RESULT, stats.getStat(newMetric));
    }
}
