/*
 * TestHelper.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.StringReader;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;


/**
 * A helper class to provide common verification methods for the tests.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class TestHelper {
    /**
     * Private constructor to prevent instantiation.
     */
    private TestHelper() {
    }

    /**
     * Verifies if the generated report specified in the reportString is exactly the same as the correct report stored
     * in the reportFile. The comparison is line-by-line based, so that the line characters would not cause a problem
     * in different OS.
     *
     * @param reportFile the file storing the correct report.
     * @param reportString the generated report to be verified.
     *
     * @return whether the generated report is correct.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public static boolean verifyReport(File reportFile, String reportString)
        throws Exception
    {
        BufferedReader reader1 = new BufferedReader(new FileReader(reportFile));
        BufferedReader reader2 = new BufferedReader(new StringReader(reportString));

        return verifyReport(reader1, reader2);
    }

    /**
     * Verifies if the generated report specified in the reportFile2 is exactly the same as the correct report stored
     * in the reportFile1. The comparison is line-by-line based, so that the line characters would not cause a problem
     * in different OS.
     *
     * @param reportFile1 the file storing the correct report.
     * @param reportFile2 the file holding the generated report.
     *
     * @return whether the generated report is correct.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public static boolean verifyReport(File reportFile1, File reportFile2)
        throws Exception
    {
        BufferedReader reader1 = new BufferedReader(new FileReader(reportFile1));
        BufferedReader reader2 = new BufferedReader(new FileReader(reportFile2));

        return verifyReport(reader1, reader2);
    }

    /**
     * Verifies if the reports pointed by the two readers are the same. The comparison is line-by-line based, so that
     * the line characters would not cause a problem in different OS.
     *
     * @param reader1 the first reader.
     * @param reader2 the second reader.
     *
     * @return whether the contents of the readers are the same.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    private static boolean verifyReport(BufferedReader reader1, BufferedReader reader2)
        throws Exception
    {
        String line1 = null;
        String line2 = null;

        while (true) {
            line1 = reader1.readLine();
            line2 = reader2.readLine();

            if ((line1 == null) || (line2 == null)) {
                break; // stop if any reader reaches the end
            }
            if (!line1.equals(line2)) {
                break; // stop if there is any difference
            }
        }
        reader1.close();
        reader2.close();

        // both readers should reach the end
        return (line1 == null) && (line2 == null);
    }

    /**
     * Checks whether the two given results are equal to each other. To be considered equal, the list of metrics and
     * the values for each metric must be the same.
     *
     * @param result1 the first stats result.
     * @param result2 the second stats result.
     *
     * @return whether the two given results are equal to each other.
     */
    public static boolean equals(StatsResult result1, StatsResult result2) {
        Set metrics1 = new HashSet(result1.getMetrics());
        Set metrics2 = new HashSet(result2.getMetrics());

        // check the list of metrics
        if (!metrics1.equals(metrics2)) {
            return false;
        }

        // check the value of each metric
        for (Iterator i = metrics1.iterator(); i.hasNext();) {
            Metric metric = (Metric) i.next();

            if (result1.getStat(metric) != result2.getStat(metric)) {
                return false;
            }
        }
        return true;
    }
}
