/*
 * XmlStatsReportingTest.java
 *
 * Copyright (c) 2005 TopCoder, Inc., All Rights Reserved
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.matcher.TextFileMatcher;
import com.topcoder.file.statistics.processor.linecounter.CStyleLineCounter;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;
import com.topcoder.file.statistics.reporting.XmlStatsReporting;

import com.topcoder.util.config.Property;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;


/**
 * Unit tests for XmlStatsReporting implementation.
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class XmlStatsReportingTest extends TestCase {
    /**
     * The file storing the correct raw XML report.
     */
    private static final String RAW_REPORT_FILE = "test_files/raw.xml";

    /**
     * The file storing the correct transformed XML report.
     */
    private static final String TRANSFORM_REPORT_FILE = "test_files/transform.xml";

    /**
     * The stylesheet file to transform the raw XML.
     */
    private static final String STYLESHEET_FILE = "test_files/stylesheet.xsl";

    /**
     * The path of the base directory used in the reporting.
     */
    private static final String BASE_DIRECTORY = "test_files/";

    /**
     * The XmlStatsReporting instances to test against.
     */
    private XmlStatsReporting[] reportings = null;

    /**
     * The base directory used in the reporting.
     */
    private File baseDirectory = null;

    /**
     * Creates a test suite for the tests in this test case.
     *
     * @return a TestSuite for this test case
     */
    public static Test suite() {
        TestSuite suite = new TestSuite(XmlStatsReportingTest.class);

        return suite;
    }

    /**
     * Prepares three XmlStatsReporting instances for testing. reportings[0] is constructed using the default
     * constructor, without specifying the stylesheet. reportings[1] is constructed with a stylesheet
     * "test_files/stylesheet.xsl". reportings[2] is constructed with property loaded with stylesheet
     * "test_files/stylesheet.xsl". Also prepares the associated base directory.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void setUp() throws Exception {
        ConfigHelper.loadConfig();

        reportings = new XmlStatsReporting[3];
        reportings[0] = new XmlStatsReporting();
        reportings[1] = new XmlStatsReporting(new File(STYLESHEET_FILE));
        reportings[2] = new XmlStatsReporting(ConfigHelper.getRootProperty(
                    ConfigHelper.GOOD_XML_STATS_REPORTING_NAMESPACE));
        baseDirectory = new File(BASE_DIRECTORY);
    }

    /**
     * Clears all the namespaces.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void tearDown() throws Exception {
        ConfigHelper.unloadConfig();
    }

    /**
     * Test of constructor with null stylesheet. Expects NullPointerException.
     */
    public void testConstructor_NullStylesheet() {
        try {
            new XmlStatsReporting((File) null);
            fail("Creates XmlStatsReporting with null stylesheet");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with stylesheet not being a valid file. Expects IllegalArgumentException.
     */
    public void testConstructor_InvalidStylesheet() {
        try {
            new XmlStatsReporting(new File("test_files/"));
            fail("Creates XmlStatsReporting with stylesheet not being a valid file");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * Test of constructor with null property. Expects NullPointerException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_NullProperty() throws Exception {
        try {
            new XmlStatsReporting((Property) null);
            fail("Creates XmlStatsReporting with null property");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * Test of constructor with bad property (stylesheet not being a valid file). Expects ConfigurationException.
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testConstructor_BadProperty() throws Exception {
        try {
            new XmlStatsReporting(ConfigHelper.getRootProperty(ConfigHelper.BAD_XML_STATS_REPORTING_NAMESPACE));
            fail("Creates XmlStatsReporting with bad property");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * Test of generateReport method. First, aggregates some results by processing the files "test_files/a/q.java" and
     * "test_files/b/test.sql" using the processors TextLineCounter and CStyleLineCounter. Verifies if reportings[0]
     * generates the raw XML report stored in "test_files/raw.xml". Verifies if reportings[1] and reportings[2]
     * generate the transformed XML report stored in "test_files/transform.xml".
     *
     * @throws Exception if any unexpected exception occurs.
     */
    public void testGenerateReport() throws Exception {
        for (int i = 0; i < reportings.length; i++) {
            reportings[i].reset(baseDirectory);

            // aggregate the result for "test_files/b/test.sql"
            FileProcessor processor = new TextLineCounter("Text File", new TextFileMatcher());
            File file = new File("test_files/b/test.sql");
            StatsResult result = processor.process(file);

            reportings[i].aggregate(processor, file, result);

            // aggregate the result for "test_files/a/q.java"
            processor = new CStyleLineCounter("Source File", new TextFileMatcher());
            file = new File("test_files/a/q.java");
            result = processor.process(file);

            reportings[i].aggregate(processor, file, result);
        }

        assertTrue("Generates an incorrect raw XML report",
            TestHelper.verifyReport(new File(RAW_REPORT_FILE), reportings[0].generateReport()));
        assertTrue("Generates an incorrect transformed XML report",
            TestHelper.verifyReport(new File(TRANSFORM_REPORT_FILE), reportings[1].generateReport()));
        assertTrue("Generates an incorrect transformed XML report",
            TestHelper.verifyReport(new File(TRANSFORM_REPORT_FILE), reportings[2].generateReport()));
    }
}
